/**
 * apeman task to render React ApStyle element as a css file.
 * @function apemanTaskApStyle
 * @param {string} src - React style source file.
 * @param {string} dest - Destination file name.
 * @param {object} [options] - Optional settings.
 * @param {string} [options.mode='644'] - File permission mode.
 * @param {string} [options.props={}] - Properties to render.
 * @returns {function} - Defined task function.
 */

'use strict'

const React = require('react')
const ReactDOM = require('react-dom/server')
const os = require('os')
const co = require('co')
const findout = require('findout')
const cssbeautify = require('cssbeautify')
const _comment = require('./_comment')
const xml2js = require('xml2js')
const writeout = require('writeout')
const pkg = require('../package.json')

/** @lends define */
function define (src, dest, options = {}) {
  let { props, mode } = options

  /**
   * Defined task.
   * @function task
   * @param {object} context - Apeman task context.
   * @returns {Promise}
   */
  function task (context) {
    let { logger } = context

    let Component = src

    if (typeof Component === 'string') {
      Component = findout(Component, { safe: true })
    }

    if (!Component) {
      logger.warn(`[${pkg.name}] src not found: ${src}`)
      return Promise.resolve()
    }

    let element = React.createElement(Component.default || Component, props || {})

    let html = ReactDOM.renderToStaticMarkup(element)

    return co(function * () {
      let data = yield new Promise((resolve, reject) =>
        xml2js.parseString(html, (err, result) =>
          err ? reject(err) : resolve(result)
        )
      )
      let { style } = data
      if (!style) {
        throw new Error('Failed to parse style with html: ' + html)
      }
      let css = [
        _comment(dest, src),
        cssbeautify(style._)
      ].join(os.EOL)
      let result = yield writeout(dest, css, {
        skipIfIdentical: true,
        mkdirp: true,
        mode: mode || '644'
      })
      if (!result.skipped) {
        console.log(`File generated: ${result.filename}`)
      }
    })
  }

  // Description of this task.
  task.$desc = 'Render react ap-style component.'
  return task
}

module.exports = define
